@@ -258,7 +258,8 @@ class BackupManager(RemoteStatusMixin):
                            len(available_backups),
                            minimum_redundancy)
             return
-
+        # Keep track of when the delete operation started.
+        delete_start_time = datetime.datetime.now()
         output.info("Deleting backup %s for server %s",
                     backup.backup_id, self.config.name)
         previous_backup = self.get_previous_backup(backup.backup_id)
@@ -315,7 +316,13 @@ class BackupManager(RemoteStatusMixin):
                          backup.get_basebackup_directory())
             return
         self.backup_cache_remove(backup)
-        output.info("Done")
+        # Save the time of the complete removal of the backup
+        delete_end_time = datetime.datetime.now()
+        output.info("Deleted backup %s (start time: %s, elapsed time: %s)",
+                    backup.backup_id,
+                    delete_start_time.ctime(),
+                    human_readable_timedelta(
+                        delete_end_time - delete_start_time))
 
     def backup(self):
         """
@@ -698,6 +705,53 @@ class BackupManager(RemoteStatusMixin):
                     '(history: %s, backup_labels: %s, wal_file: %s)',
                     self.config.name, history_count, label_count, wal_count)
 
+    def get_latest_archived_wals_info(self):
+        """
+        Return a dictionary of timelines associated with the
+        WalFileInfo of the last WAL file in the archive,
+        or None if the archive doesn't contain any WAL file.
+
+        :rtype: dict[str, WalFileInfo]|None
+        """
+        from os.path import isdir, join
+
+        root = self.config.wals_directory
+
+        # If the WAL archive directory doesn't exists the archive is empty
+        if not isdir(root):
+            return None
+
+        # Traverse all the directory in the archive in reverse order,
+        # returning the first WAL file found
+        timelines = {}
+        for name in sorted(os.listdir(root), reverse=True):
+            fullname = join(root, name)
+            # All relevant files are in subdirectories, so
+            # we skip any non-directory entry
+            if isdir(fullname):
+                # Extract the timeline. If it is not valid, skip this directory
+                try:
+                    timeline = name[0:8]
+                    int(timeline, 16)
+                except ValueError:
+                    continue
+
+                # If this timeline already has a file, skip this directory
+                if timeline in timelines:
+                    continue
+
+                hash_dir = fullname
+                # Inspect contained files in reverse order
+                for wal_name in sorted(os.listdir(hash_dir), reverse=True):
+                    fullname = join(hash_dir, wal_name)
+                    # Return the first file that has the correct name
+                    if not isdir(fullname) and xlog.is_wal_file(fullname):
+                        timelines[timeline] = WalFileInfo.from_file(fullname)
+                        break
+
+        # Return the timeline map or None if it is empty
+        return timelines or None
+
     def remove_wal_before_backup(self, backup_info, timelines_to_protect=None):
         """
         Remove WAL files which have been archived before the start of
