@@ -25,11 +25,15 @@ import dateutil.tz
 
 from barman import xlog
 from barman.compression import identify_compression
+from barman.exceptions import BackupInfoBadInitialisation
 
-# create a namedtuple object called Tablespace with 'name' 'oid' and 'location'
+# Named tuple representing a Tablespace with 'name' 'oid' and 'location'
 # as property.
 Tablespace = collections.namedtuple('Tablespace', 'name oid location')
 
+# Named tuple representing a file 'path' with an associated 'file_type'
+TypedFile = collections.namedtuple('ConfFile', 'file_type path')
+
 _logger = logging.getLogger(__name__)
 
 
@@ -393,18 +397,6 @@ class WalFileInfo(FieldListFile):
         return os.path.join(server.config.wals_directory, self.relpath())
 
 
-class UnknownBackupIdException(Exception):
-    """
-    The searched backup_id doesn't exists
-    """
-
-
-class BackupInfoBadInitialisation(Exception):
-    """
-    Exception for a bad initialization error
-    """
-
-
 class BackupInfo(FieldListFile):
 
     #: Conversion to string
@@ -473,7 +465,7 @@ class BackupInfo(FieldListFile):
         self.config = server.config
         self.backup_manager = self.server.backup_manager
         self.server_name = self.config.name
-        self.mode = self.backup_manager.name
+        self.mode = self.backup_manager.mode
         if backup_id:
             # Cannot pass both info_file and backup_id
             if info_file:
@@ -557,7 +549,7 @@ class BackupInfo(FieldListFile):
         If tablespace_oid is passed, build the path to the tablespace
         base directory, according with the backup version
 
-        :param str tablespace_oid: the oid of a valid tablespace
+        :param int tablespace_oid: the oid of a valid tablespace
         """
         # Check if a tablespace oid is passed and if is a valid oid
         if tablespace_oid is not None and (
@@ -586,6 +578,36 @@ class BackupInfo(FieldListFile):
         # Return the built path
         return os.path.join(*path)
 
+    def get_external_config_files(self):
+        """
+        Identify all the configuration files that reside outside the PGDATA.
+
+        Returns a list of TypedFile objects.
+
+        :rtype: list[TypedFile]
+        """
+
+        config_files = []
+        for file_type in ('config_file', 'hba_file', 'ident_file'):
+            config_file = getattr(self, file_type, None)
+            if config_file:
+                # Consider only those that reside outside of the original
+                # PGDATA directory
+                if config_file.startswith(self.pgdata):
+                    _logger.debug("Config file '%s' already in PGDATA",
+                                  config_file[len(self.pgdata) + 1:])
+                    continue
+                config_files.append(TypedFile(file_type, config_file))
+        # Check for any include directives in PostgreSQL configuration
+        # Currently, include directives are not supported for files that
+        # reside outside PGDATA. These files must be manually backed up.
+        # Barman will emit a warning and list those files
+        if self.included_files:
+            for included_file in self.included_files:
+                if not included_file.startswith(self.pgdata):
+                    config_files.append(TypedFile('include', included_file))
+        return config_files
+
     def get_filename(self):
         """
         Get the default filename for the backup.info file based on
