@@ -194,7 +194,13 @@ class PostgreSQL(with_metaclass(ABCMeta, RemoteStatusMixin)):
             major = int(conn.server_version / 10000)
             minor = int(conn.server_version / 100 % 100)
             patch = int(conn.server_version % 100)
-            return "%d.%d.%d" % (major, minor, patch)
+            if major < 10:
+                return "%d.%d.%d" % (major, minor, patch)
+            if minor != 0:
+                _logger.warning(
+                    "Unexpected non zero minor version %s in %s",
+                    minor, conn.server_version)
+            return "%d.%d" % (major, patch)
         except PostgresConnectionError as e:
             _logger.debug("Error retrieving PostgreSQL version: %s",
                           str(e).strip())
@@ -566,7 +572,7 @@ class PostgreSQLConnection(PostgreSQL):
                 "OR last_failed_wal LIKE '%.history' "
                 "AND substring(last_failed_wal from 1 for 8) "
                 "<= substring(last_archived_wal from 1 for 8) "
-                "OR last_failed_wal <= last_archived_wal) "
+                "OR last_failed_time <= last_archived_time) "
                 "AS is_archiving, "
                 "CAST (archived_count AS NUMERIC) "
                 "/ EXTRACT (EPOCH FROM age(now(), stats_reset)) "
@@ -1004,9 +1010,8 @@ class PostgreSQLConnection(PostgreSQL):
 
         To be SURE of the switch of a xlog, we collect the xlogfile name
         before and after the switch.
-        If the name of the xlog file post switch is greater than the one
-        collected before the switch have been executed.
-        Returns an empty string otherwise.
+        The method returns the just closed xlog file name if the current xlog
+        file has changed, it returns an empty string otherwise.
 
         The method returns None if something went wrong during the execution
         of the pg_switch_xlog command.
@@ -1035,7 +1040,7 @@ class PostgreSQLConnection(PostgreSQL):
                         'pg_current_xlog_insert_location())')
             post_switch = cur.fetchone()[0]
             if pre_switch < post_switch:
-                return post_switch
+                return pre_switch
             else:
                 return ''
         except (PostgresConnectionError, psycopg2.Error) as e:
