@@ -455,7 +455,6 @@ class RecoveryExecutor(object):
 
             try:
                 # remove the current link in pg_tblspc, if it exists
-                # (raise an exception if it is a directory)
                 cmd.delete_if_exists(pg_tblspc_file)
                 # create tablespace location, if does not exist
                 # (raise an exception if it is not possible)
@@ -930,18 +929,26 @@ class RecoveryExecutor(object):
         commenting out the given settings, and adding the ones generated by
         Barman.
 
-        If backup_filename is passed, performs a backup copy first.
+        If backup_filename is passed, keep a backup copy.
 
         :param filename: the PostgreSQL configuration file
         :param settings: dictionary of settings to be mangled
         :param backup_filename: config file backup copy. Default is None.
         """
-        if backup_filename:
-            shutil.copy2(filename, backup_filename)
-
+        # Read the full content of the file in memory
         with open(filename) as f:
             content = f.readlines()
 
+        # Rename the original file to backup_filename or to a temporary name
+        # if backup_filename is missing. We need to keep it to preserve
+        # permissions.
+        if backup_filename:
+            orig_filename = backup_filename
+        else:
+            orig_filename = "%s.config_mangle.old" % filename
+        shutil.move(filename, orig_filename)
+
+        # Write the mangled content
         mangled = []
         with open(filename, 'w') as f:
             for l_number, line in enumerate(content):
@@ -962,6 +969,14 @@ class RecoveryExecutor(object):
                         continue
                 f.write(line)
 
+        # Restore original permissions
+        shutil.copymode(orig_filename, filename)
+
+        # If a backup copy of the file is not requested,
+        # unlink the orig file
+        if not backup_filename:
+            os.unlink(orig_filename)
+
         return mangled
 
     def _pg_config_detect_possible_issues(self, filename):
