@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2015 2ndQuadrant Italia Srl
+# Copyright (C) 2011-2016 2ndQuadrant Italia Srl
 #
 # This file is part of Barman.
 #
@@ -17,7 +17,8 @@
 """
 This module defines backup retention policies. A backup retention
 policy in Barman is a user-defined policy for determining how long
-backups and archived logs (WAL segments) need to be retained for media recovery.
+backups and archived logs (WAL segments) need to be retained for media
+recovery.
 You can define a retention policy in terms of backup redundancy
 or a recovery window.
 Barman retains the periodical backups required to satisfy
@@ -25,19 +26,21 @@ the current retention policy, and any archived WAL files required for complete
 recovery of those backups.
 """
 
+import logging
+import re
 from abc import ABCMeta, abstractmethod
 from datetime import datetime, timedelta
-import re
-import logging
-from barman.infofile import BackupInfo
+
 from dateutil import tz
 
+from barman.infofile import BackupInfo
+from barman.utils import with_metaclass
+
 _logger = logging.getLogger(__name__)
 
 
-class RetentionPolicy(object):
+class RetentionPolicy(with_metaclass(ABCMeta, object)):
     """Abstract base class for retention policies"""
-    __metaclass__ = ABCMeta
 
     def __init__(self, mode, unit, value, context, server):
         """Constructor of the retention policy base class"""
@@ -55,7 +58,8 @@ class RetentionPolicy(object):
             context = self.context
         # Overrides the list of available backups
         if source is None:
-            source = self.server.get_available_backups(BackupInfo.STATUS_NOT_EMPTY)
+            source = self.server.get_available_backups(
+                BackupInfo.STATUS_NOT_EMPTY)
         if context == 'BASE':
             return self._backup_report(source)
         elif context == 'WAL':
@@ -124,8 +128,8 @@ class RetentionPolicy(object):
 class RedundancyRetentionPolicy(RetentionPolicy):
     """
     Retention policy based on redundancy, the setting that determines
-    many periodical backups to keep. A redundancy-based retention polic
-    yis contrasted with retention policy that uses a recovery window.
+    many periodical backups to keep. A redundancy-based retention policy
+    is contrasted with retention policy that uses a recovery window.
     """
 
     _re = re.compile(r'^\s*redundancy\s+(\d+)\s*$', re.IGNORECASE)
@@ -160,7 +164,7 @@ class RedundancyRetentionPolicy(RetentionPolicy):
         # Non DONE backups are classified as NONE
         # NOTE: reverse key orders (simulate reverse chronology)
         i = 0
-        for bid in sorted(backups.iterkeys(), reverse=True):
+        for bid in sorted(backups.keys(), reverse=True):
             if backups[bid].status == BackupInfo.DONE:
                 if i < redundancy:
                     report[bid] = BackupInfo.VALID
@@ -189,13 +193,14 @@ class RedundancyRetentionPolicy(RetentionPolicy):
 class RecoveryWindowRetentionPolicy(RetentionPolicy):
     """
     Retention policy based on recovery window. The DBA specifies a period of
-    time and Barman ensures retention of backups and archived WAL files required
-    for point-in-time recovery to any time during the recovery window.
+    time and Barman ensures retention of backups and archived WAL files
+    required for point-in-time recovery to any time during the recovery window.
     The interval always ends with the current time and extends back in time
     for the number of days specified by the user.
-    For example, if the retention policy is set for a recovery window of seven days,
-    and the current time is 9:30 AM on Friday, Barman retains the backups required
-    to allow point-in-time recovery back to 9:30 AM on the previous Friday.
+    For example, if the retention policy is set for a recovery window of
+    seven days, and the current time is 9:30 AM on Friday, Barman retains
+    the backups required to allow point-in-time recovery back to 9:30 AM
+    on the previous Friday.
     """
 
     _re = re.compile(
@@ -249,7 +254,7 @@ class RecoveryWindowRetentionPolicy(RetentionPolicy):
         found = False
         valid = 0
         # NOTE: reverse key orders (simulate reverse chronology)
-        for bid in sorted(backups.iterkeys(), reverse=True):
+        for bid in sorted(backups.keys(), reverse=True):
             # We are interested in DONE backups only
             if backups[bid].status == BackupInfo.DONE:
                 if found:
