@@ -25,6 +25,7 @@ import logging
 import os
 import shutil
 import sys
+import time
 from collections import namedtuple
 from contextlib import contextmanager
 from tempfile import NamedTemporaryFile
@@ -1734,11 +1735,12 @@ class Server(RemoteStatusMixin):
                          'no such process for server %s',
                          task, self.config.name)
 
-    def switch_xlog(self, force=False):
+    def switch_xlog(self, force=False, archive=None, archive_timeout=None):
         """
         Execute the switch-xlog command on the target server
         """
         try:
+
             if force:
                 # If called with force, execute a checkpoint before the
                 # switch_xlog command
@@ -1747,18 +1749,42 @@ class Server(RemoteStatusMixin):
 
             # Perform the switch_xlog. expect a WAL name only if the switch
             # has been successfully executed, False otherwise.
-            switch_xlogfile = self.postgres.switch_xlog()
-            if switch_xlogfile is None:
+            closed_xlog = self.postgres.switch_xlog()
+            if closed_xlog is None:
                 # Something went wrong during the execution of the
                 # pg_switch_xlog command
                 output.error("Unable to perform pg_switch_xlog "
                              "for server '%s'." % self.config.name)
                 return
-            if switch_xlogfile:
+            if closed_xlog:
                 # The switch_xlog command have been executed successfully
                 output.info(
-                    "Switch to %s for server '%s'" %
-                    (switch_xlogfile, self.config.name))
+                    "The xlog file %s has been closed on server '%s'" %
+                    (closed_xlog, self.config.name))
+                # If the user has asked to wait for a WAL file to be archived,
+                # wait until a new WAL file has been found
+                # or the timeout has expired
+                if archive:
+                    output.info(
+                        "Waiting for the xlog file %s from server '%s' "
+                        "(max: %s seconds)",
+                        closed_xlog, self.config.name, archive_timeout)
+                    # Wait for a new file until end_time
+                    end_time = time.time() + archive_timeout
+                    while time.time() < end_time:
+                        self.backup_manager.archive_wal(verbose=False)
+
+                        # Finish if the closed wal file is in the archive.
+                        if os.path.exists(self.get_wal_full_path(closed_xlog)):
+                                break
+
+                        # sleep a bit before retrying
+                        time.sleep(.1)
+                    else:
+                        output.error("The xlog file %s has not been received "
+                                     "in %s seconds",
+                                     closed_xlog, archive_timeout)
+
             else:
                 # Is not necessary to perform a switch_xlog
                 output.info("No switch required for server '%s'" %
@@ -1818,8 +1844,13 @@ class Server(RemoteStatusMixin):
             if not os.path.exists(history_path):
                 break
 
-            # Get content of the file
-            history_info = xlog.decode_history_file(history_path)
+            # Create the WalFileInfo object using the file
+            wal_info = WalFileInfo.from_file(history_path)
+            # Get content of the file. We need to pass a compressor manager
+            # here to handle an eventual compression of the history file
+            history_info = xlog.decode_history_file(
+                wal_info,
+                self.backup_manager.compression_manager)
 
             # Save the history only if is reachable from this timeline.
             for tinfo in history_info:
