@@ -28,6 +28,7 @@ import logging
 import logging.handlers
 import os
 import pwd
+import re
 import signal
 from contextlib import contextmanager
 
@@ -154,9 +155,10 @@ def human_readable_timedelta(timedelta):
     delta = abs(timedelta)
     # Calculate time units for the given interval
     time_map = {
-        'day': int(delta.days % 365),
+        'day': int(delta.days),
         'hour': int(delta.seconds / 3600),
         'minute': int(delta.seconds / 60) % 60,
+        'second': int(delta.seconds % 60),
     }
     # Build the resulting string
     time_list = []
@@ -182,7 +184,19 @@ def human_readable_timedelta(timedelta):
         else:
             time_list.append('%s minutes' % time_map['minute'])
 
+    # 'Second' part
+    if time_map['second'] > 0:
+        if time_map['second'] == 1:
+            time_list.append('%s second' % time_map['second'])
+        else:
+            time_list.append('%s seconds' % time_map['second'])
+
     human = ', '.join(time_list)
+
+    # Take care of timedelta when is shorter than a second
+    if delta < datetime.timedelta(seconds=1):
+        human = 'less than one second'
+
     # If timedelta is negative append 'ago' suffix
     if delta != timedelta:
         human += " ago"
@@ -275,7 +289,7 @@ def fsync_dir(dir_path):
 
 def simplify_version(version_string):
     """
-    Simplify a version number using a major.minor format
+    Simplify a version number by removing the patch level
 
     :param version_string: the version number to simplify
     :return str: the simplified version number
@@ -283,7 +297,14 @@ def simplify_version(version_string):
     if version_string is None:
         return None
     version = version_string.split('.')
-    return '.'.join(version[:2])
+    # If a development/beta/rc version, split out the string part
+    unreleased = re.search(r'[^0-9.]', version[-1])
+    if unreleased:
+        last_component = version.pop()
+        number = last_component[:unreleased.start()]
+        string = last_component[unreleased.start():]
+        version += [number, string]
+    return '.'.join(version[:-1])
 
 
 def with_metaclass(meta, *bases):
