@@ -17,20 +17,21 @@
 
 import collections
 import datetime
+import errno
 import filecmp
 import logging
 import os
 import shutil
 from abc import ABCMeta, abstractmethod
-from distutils.version import LooseVersion as Version
 from glob import glob
 
-from barman import output, utils, xlog
-from barman.backup import DuplicateWalFile, MatchingDuplicateWalFile
-from barman.command_wrappers import (Command, CommandFailedException,
-                                     PgReceiveXlog)
-from barman.hooks import (AbortedRetryHookScript, HookScriptRunner,
-                          RetryHookScriptRunner)
+from distutils.version import LooseVersion as Version
+
+from barman import output, xlog
+from barman.command_wrappers import CommandFailedException, PgReceiveXlog
+from barman.exceptions import (AbortedRetryHookScript, ArchiverFailure,
+                               DuplicateWalFile, MatchingDuplicateWalFile)
+from barman.hooks import HookScriptRunner, RetryHookScriptRunner
 from barman.infofile import WalFileInfo
 from barman.remote_status import RemoteStatusMixin
 from barman.utils import fsync_dir, mkpath, with_metaclass
@@ -38,33 +39,62 @@ from barman.utils import fsync_dir, mkpath, with_metaclass
 _logger = logging.getLogger(__name__)
 
 
-class WalArchiverBatch(list):
-    def __init__(self, items, errors=None, skip=None):
+class WalArchiverQueue(list):
+    def __init__(self, items, errors=None, skip=None, batch_size=0):
         """
-        A WalArchiverBatch is a list of WalFileInfo which has two extra
+        A WalArchiverQueue is a list of WalFileInfo which has two extra
         attribute list:
 
         * errors: containing a list of unrecognized files
         * skip: containing a list of skipped files.
 
+        It also stores batch run size information in case
+        it is requested by configuration, in order to limit the
+        number of WAL files that are processed in a single
+        run of the archive-wal command.
+
         :param items: iterable from which initialize the list
+        :param batch_size: size of the current batch run (0=unlimited)
         :param errors: an optional list of unrecognized files
         :param skip: an optional list of skipped files
         """
-        super(WalArchiverBatch, self).__init__(items)
+        super(WalArchiverQueue, self).__init__(items)
         self.skip = []
         self.errors = []
         if skip is not None:
             self.skip = skip
         if errors is not None:
             self.errors = errors
+        # Normalises batch run size
+        if batch_size > 0:
+            self.batch_size = batch_size
+        else:
+            self.batch_size = 0
 
+    @property
+    def size(self):
+        """
+        Number of valid WAL segments waiting to be processed (in total)
 
-class ArchiverFailure(Exception):
-    """
-    Exception representing a failure during the execution
-    of the archive process
-    """
+        :return int: total number of valid WAL files
+        """
+        return len(self)
+
+    @property
+    def run_size(self):
+        """
+        Number of valid WAL files to be processed in this run - takes
+        in consideration the batch size
+
+        :return int: number of valid WAL files for this batch run
+        """
+        # In case a batch size has been explicitly specified
+        # (i.e. batch_size > 0), returns the minimum number between
+        # batch size and the queue size. Otherwise, simply
+        # returns the total queue size (unlimited batch size).
+        if self.batch_size > 0:
+            return min(self.size, self.batch_size)
+        return self.size
 
 
 class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
@@ -105,24 +135,62 @@ class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
         """
         compressor = self.backup_manager.compression_manager.get_compressor()
         stamp = datetime.datetime.utcnow().strftime('%Y%m%dT%H%M%SZ')
-        found = False
-        if verbose:
-            output.info("Processing xlog segments from %s for %s",
-                        self.name,
-                        self.config.name,
-                        log=False)
+        processed = 0
+        header = "Processing xlog segments from %s for %s" % (
+                 self.name, self.config.name)
+
+        # Get the next batch of WAL files to be processed
         batch = self.get_next_batch()
+
+        # Analyse the batch and properly log the information
+        if batch.size:
+            if batch.size > batch.run_size:
+                # Batch mode enabled
+                _logger.info("Found %s xlog segments from %s for %s."
+                             " Archive a batch of %s segments in this run.",
+                             batch.size,
+                             self.name,
+                             self.config.name,
+                             batch.run_size)
+                header += " (batch size: %s)" % batch.run_size
+            else:
+                # Single run mode (traditional)
+                _logger.info("Found %s xlog segments from %s for %s."
+                             " Archive all segments in one run.",
+                             batch.size,
+                             self.name,
+                             self.config.name)
+        else:
+            _logger.info("No xlog segments found from %s for %s.",
+                         self.name,
+                         self.config.name)
+
+        # Print the header (verbose mode)
+        if verbose:
+            output.info(header, log=False)
+
+        # Loop through all available WAL files
         for wal_info in batch:
-            if not found and not verbose:
-                output.info("Processing xlog segments from %s for %s",
-                            self.name,
-                            self.config.name,
-                            log=False)
-            found = True
+            # Print the header (non verbose mode)
+            if not processed and not verbose:
+                output.info(header, log=False)
+
+            # Exit when archive batch size is reached
+            if processed >= batch.run_size:
+                _logger.debug("Batch size reached (%s) - "
+                              "Exit %s process for %s",
+                              batch.batch_size,
+                              self.name,
+                              self.config.name)
+                break
+
+            processed += 1
 
             # Report to the user the WAL file we are archiving
             output.info("\t%s", wal_info.name, log=False)
-            _logger.info("Archiving %s/%s", self.config.name, wal_info.name)
+            _logger.info("Archiving segment %s of %s from %s: %s/%s",
+                         processed, batch.run_size, self.name,
+                         self.config.name, wal_info.name)
             # Archive the WAL file
             try:
                 self.archive_wal(compressor, wal_info)
@@ -156,20 +224,39 @@ class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
             # flush and fsync for every line
             fxlogdb.flush()
             os.fsync(fxlogdb.fileno())
-        if not found and verbose:
+
+        if processed:
+            _logger.debug("Archived %s out of %s xlog segments from %s for %s",
+                          processed, batch.size, self.name, self.config.name)
+        elif verbose:
             output.info("\tno file found", log=False)
+
         if batch.errors:
             output.info("Some unknown objects have been found while "
                         "processing xlog segments for %s. "
                         "Objects moved to errors directory:",
                         self.config.name,
                         log=False)
+            # Log unexpected files
+            _logger.warning("Archiver is about to move %s unexpected file(s) "
+                            "to errors directory for %s from %s",
+                            len(batch.errors),
+                            self.config.name,
+                            self.name)
             for error in batch.errors:
-                output.info("\t%s", error)
+                basename = os.path.basename(error)
+                output.info("\t%s", basename, log=False)
+                # Print informative log line.
+                _logger.warning("Moving unexpected file for %s from %s: %s",
+                                self.config.name, self.name, basename)
                 error_dst = os.path.join(
                     self.config.errors_directory,
-                    "%s.%s.unknown" % (os.path.basename(error), stamp))
-                shutil.move(error, error_dst)
+                    "%s.%s.unknown" % (basename, stamp))
+                try:
+                    shutil.move(error, error_dst)
+                except IOError as e:
+                    if e.errno == errno.ENOENT:
+                        _logger.warning('%s not found' % error)
 
     def archive_wal(self, compressor, wal_info):
         """
@@ -295,9 +382,9 @@ class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
     @abstractmethod
     def get_next_batch(self):
         """
-        Return a WalArchiverBatch containing the WAL files to be archived.
+        Return a WalArchiverQueue containing the WAL files to be archived.
 
-        :rtype: WalArchiverBatch
+        :rtype: WalArchiverQueue
         """
 
     @abstractmethod
@@ -310,6 +397,12 @@ class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
              of the results of the various checks
         """
 
+    @abstractmethod
+    def status(self):
+        """
+        Set additional status info - invoked by Server.status()
+        """
+
     @staticmethod
     def summarise_error_files(error_files):
         """
@@ -360,7 +453,7 @@ class FileWalArchiver(WalArchiver):
         """
         result = dict.fromkeys(
             ['archive_mode', 'archive_command'], None)
-        postgres = self.backup_manager.server.postgres
+        postgres = self.server.postgres
         # If Postgres is not available we cannot detect anything
         if not postgres:
             return result
@@ -380,8 +473,10 @@ class FileWalArchiver(WalArchiver):
         Returns the next batch of WAL files that have been archived through
         a PostgreSQL's 'archive_command' (in the 'incoming' directory)
 
-        :return: WalArchiverBatch: list of WAL files
+        :return: WalArchiverQueue: list of WAL files
         """
+        # Get the batch size from configuration (0 = unlimited)
+        batch_size = self.config.archiver_batch_size
         # List and sort all files in the incoming directory
         file_names = glob(os.path.join(
             self.config.incoming_wals_directory, '*'))
@@ -399,7 +494,9 @@ class FileWalArchiver(WalArchiver):
 
         # Build the list of WalFileInfo
         wal_files = [WalFileInfo.from_file(f) for f in files]
-        return WalArchiverBatch(wal_files, errors=errors)
+        return WalArchiverQueue(wal_files,
+                                batch_size=batch_size,
+                                errors=errors)
 
     def check(self, check_strategy):
         """
@@ -439,6 +536,47 @@ class FileWalArchiver(WalArchiver):
                 self.config.name, 'archive_command', False,
                 'please set it accordingly to documentation')
 
+    def status(self):
+        """
+        Set additional status info - invoked by Server.status()
+        """
+        remote_status = self.get_remote_status()
+
+        # If archive_mode is None, there are issues connecting to PostgreSQL
+        if remote_status['archive_mode'] is None:
+            return
+
+        output.result('status', self.config.name,
+                      "archive_command",
+                      "PostgreSQL 'archive_command' setting",
+                      remote_status['archive_command'] or
+                      "FAILED (please set it accordingly to documentation)")
+        last_wal = remote_status.get('last_archived_wal')
+        # If PostgreSQL is >= 9.4 we have the last_archived_time
+        if last_wal and remote_status.get('last_archived_time'):
+                last_wal += ", at %s" % (
+                    remote_status['last_archived_time'].ctime())
+        output.result('status', self.config.name,
+                      "last_archived_wal",
+                      "Last archived WAL",
+                      last_wal or "No WAL segment shipped yet")
+        # Set output for WAL archive failures (PostgreSQL >= 9.4)
+        if remote_status.get('failed_count') is not None:
+            remote_fail = str(remote_status['failed_count'])
+            if int(remote_status['failed_count']) > 0:
+                remote_fail += " (%s at %s)" % (
+                    remote_status['last_failed_wal'],
+                    remote_status['last_failed_time'].ctime())
+            output.result('status', self.config.name, 'failed_count',
+                          'Failures of WAL archiver', remote_fail)
+        # Add hourly archive rate if available (PostgreSQL >= 9.4) and > 0
+        if remote_status.get('current_archived_wals_per_second'):
+            output.result(
+                'status', self.config.name,
+                'server_archived_wals_per_hour',
+                'Server WAL archiving rate', '%0.2f/hour' % (
+                    3600 * remote_status['current_archived_wals_per_second']))
+
 
 class StreamingWalArchiver(WalArchiver):
     """
@@ -457,71 +595,70 @@ class StreamingWalArchiver(WalArchiver):
 
         :rtype: dict[str, None|str]
         """
-        result = dict.fromkeys(
+        remote_status = dict.fromkeys(
             ('pg_receivexlog_compatible',
              'pg_receivexlog_installed',
              'pg_receivexlog_path',
+             'pg_receivexlog_supports_slots',
+             'pg_receivexlog_synchronous',
              'pg_receivexlog_version'),
             None)
 
-        # Check the server version from the streaming
-        # connection
-        streaming = self.backup_manager.server.streaming
-        server_txt_version = None
-        if streaming:
-            server_txt_version = streaming.server_txt_version
-        if server_txt_version:
-            pg_version = Version(utils.simplify_version(server_txt_version))
-        else:
-            # No log here, it has already been logged in the
-            # StreamingConnection class or during the Server initialization
-            pg_version = None
-
-        # Detect a pg_receivexlog executable
-        pg_receivexlog = utils.which("pg_receivexlog",
-                                     self.backup_manager.server.path)
-
         # Test pg_receivexlog existence
-        if pg_receivexlog:
-            result["pg_receivexlog_installed"] = True
-            result["pg_receivexlog_path"] = pg_receivexlog
+        version_info = PgReceiveXlog.get_version_info(
+            self.server.path)
+        if version_info['full_path']:
+            remote_status["pg_receivexlog_installed"] = True
+            remote_status["pg_receivexlog_path"] = version_info['full_path']
+            remote_status["pg_receivexlog_version"] = (
+                version_info['full_version'])
+            pgreceivexlog_version = version_info['major_version']
         else:
-            result["pg_receivexlog_installed"] = False
-            return result
+            remote_status["pg_receivexlog_installed"] = False
+            return remote_status
 
-        receivexlog = Command(pg_receivexlog, check=True)
-
-        # Obtain the `pg_receivexlog` version
-        try:
-            receivexlog("--version")
-            splitter_version = receivexlog.out.strip().split()
-            result["pg_receivexlog_version"] = splitter_version[-1]
-            receivexlog_version = Version(
-                utils.simplify_version(result["pg_receivexlog_version"]))
-        except CommandFailedException as e:
-            receivexlog_version = None
-            _logger.debug("Error invoking pg_receivexlog: %s", e)
+        # Retrieve the PostgreSQL version
+        pg_version = None
+        if self.server.streaming is not None:
+            pg_version = self.server.streaming.server_major_version
 
         # If one of the version is unknown we cannot compare them
-        if receivexlog_version is None or pg_version is None:
-            return result
+        if pgreceivexlog_version is None or pg_version is None:
+            return remote_status
+
+        # pg_version is not None so transform into a Version object
+        # for easier comparison between versions
+        pg_version = Version(pg_version)
+
+        # Set conservative default values (False) for modern features
+        remote_status["pg_receivexlog_compatible"] = False
+        remote_status['pg_receivexlog_supports_slots'] = False
+        remote_status["pg_receivexlog_synchronous"] = False
 
         # pg_receivexlog 9.2 is compatible only with PostgreSQL 9.2.
-        if "9.2" == pg_version == receivexlog_version:
-            result["pg_receivexlog_compatible"] = True
+        if "9.2" == pg_version == pgreceivexlog_version:
+            remote_status["pg_receivexlog_compatible"] = True
 
         # other versions are compatible with lesser versions of PostgreSQL
         # WARNING: The development versions of `pg_receivexlog` are considered
         # higher than the stable versions here, but this is not an issue
         # because it accepts everything that is less than
         # the `pg_receivexlog` version(e.g. '9.6' is less than '9.6devel')
-        elif "9.2" < pg_version <= receivexlog_version:
-            result["pg_receivexlog_compatible"] = True
+        elif "9.2" < pg_version <= pgreceivexlog_version:
+            # At least PostgreSQL 9.3 is required here
+            remote_status["pg_receivexlog_compatible"] = True
 
-        else:
-            result["pg_receivexlog_compatible"] = False
+            # replication slots are supported starting from version 9.4
+            if "9.4" <= pg_version <= pgreceivexlog_version:
+                remote_status['pg_receivexlog_supports_slots'] = True
 
-        return result
+            # Synchronous WAL streaming requires replication slots
+            # and pg_receivexlog >= 9.5
+            if "9.4" <= pg_version and "9.5" <= pgreceivexlog_version:
+                remote_status["pg_receivexlog_synchronous"] = (
+                    self._is_synchronous())
+
+        return remote_status
 
     def receive_wal(self, reset=False):
         """
@@ -540,11 +677,11 @@ class StreamingWalArchiver(WalArchiver):
             return
 
         # Execute basic sanity checks on PostgreSQL connection
-        postgres_status = self.server.streaming.get_remote_status()
-        if postgres_status["streaming_supported"] is None:
+        streaming_status = self.server.streaming.get_remote_status()
+        if streaming_status["streaming_supported"] is None:
             raise ArchiverFailure(
                 'failed opening the PostgreSQL streaming connection')
-        elif not postgres_status["streaming_supported"]:
+        elif not streaming_status["streaming_supported"]:
             raise ArchiverFailure(
                 'PostgreSQL version too old (%s < 9.2)' %
                 self.server.streaming.server_txt_version)
@@ -558,6 +695,28 @@ class StreamingWalArchiver(WalArchiver):
                 'pg_receivexlog version not compatible with '
                 'PostgreSQL server version')
 
+        # Execute sanity check on replication slot usage
+        if self.config.slot_name:
+            # Check if slots are supported
+            if not remote_status['pg_receivexlog_supports_slots']:
+                raise ArchiverFailure(
+                    'Physical replication slot not supported by %s '
+                    '(9.4 or higher is required)' %
+                    self.server.streaming.server_txt_version)
+            # Check if the required slot exists
+            postgres_status = self.server.postgres.get_remote_status()
+            if postgres_status['replication_slot'] is None:
+                raise ArchiverFailure(
+                    "replication slot '%s' doesn't exist. "
+                    "Please execute "
+                    "'barman receive-wal --create-slot %s'" %
+                    (self.config.slot_name, self.config.name))
+            # Check if the required slot is available
+            if postgres_status['replication_slot'].active:
+                raise ArchiverFailure(
+                    "replication slot '%s' is already in use" %
+                    (self.config.slot_name,))
+
         # Make sure we are not wasting precious PostgreSQL resources
         self.server.close()
 
@@ -565,42 +724,34 @@ class StreamingWalArchiver(WalArchiver):
         try:
             output_handler = PgReceiveXlog.make_output_handler(
                 self.config.name + ': ')
-            if remote_status['pg_receivexlog_version'] >= "9.3":
-                conn_string = self.config.streaming_conninfo
-                # Set a default application_name unless defined by user
-                if ('application_name' not in
-                        self.server.streaming.conn_parameters):
-                    conn_string += ' application_name=%s' % (
-                        self.config.streaming_archiver_name
-                    )
-                # If pg_receivexlog version is >= 9.3 we use the connection
-                # string because allows the user to use all the parameters
-                # supported by the libpq library to create a connection
-                receive = PgReceiveXlog(
-                    remote_status['pg_receivexlog_path'],
-                    conn_string=conn_string,
-                    dest=self.config.streaming_wals_directory,
-                    out_handler=output_handler,
-                    err_handler=output_handler)
-            else:
-                # 9.2 version of pg_receivexlog doesn't support
-                # connection strings so the 'split' version of the conninfo
-                # option is used instead.
-                conn_params = self.server.streaming.conn_parameters
-                receive = PgReceiveXlog(
-                    remote_status['pg_receivexlog_path'],
-                    host=conn_params.get('host', None),
-                    port=conn_params.get('port', None),
-                    user=conn_params.get('user', None),
-                    dest=self.config.streaming_wals_directory,
-                    out_handler=output_handler,
-                    err_handler=output_handler)
+            receive = PgReceiveXlog(
+                connection=self.server.streaming,
+                destination=self.config.streaming_wals_directory,
+                command=remote_status['pg_receivexlog_path'],
+                version=remote_status['pg_receivexlog_version'],
+                app_name=self.config.streaming_archiver_name,
+                path=self.server.path,
+                slot_name=self.config.slot_name,
+                synchronous=remote_status['pg_receivexlog_synchronous'],
+                out_handler=output_handler,
+                err_handler=output_handler
+            )
             # Finally execute the pg_receivexlog process
             receive.execute()
         except CommandFailedException as e:
-            _logger.error(e)
-            raise ArchiverFailure("pg_receivexlog exited with an error. "
-                                  "Check the logs for more information.")
+            # Retrieve the return code from the exception
+            ret_code = e.args[0]['ret']
+            if ret_code < 0:
+                # If the return code is negative, then pg_receivexlog
+                # was terminated by a signal
+                msg = "pg_receivexlog terminated by signal: %s" \
+                      % abs(ret_code)
+            else:
+                # Otherwise terminated with an error
+                msg = "pg_receivexlog terminated with error code: %s"\
+                      % ret_code
+
+            raise ArchiverFailure(msg)
         except KeyboardInterrupt:
             # This is a normal termination, so there is nothing to do beside
             # informing the user.
@@ -626,30 +777,37 @@ class StreamingWalArchiver(WalArchiver):
         because the 'pg_receivexlog' process needs at least one file to
         detect the current streaming position after a restart.
 
-        :return: WalArchiverBatch: list of WAL files
+        :return: WalArchiverQueue: list of WAL files
         """
+        # Get the batch size from configuration (0 = unlimited)
+        batch_size = self.config.streaming_archiver_batch_size
         # List and sort all files in the incoming directory
         file_names = glob(os.path.join(
             self.config.streaming_wals_directory, '*'))
         file_names.sort()
 
         # Process anything that looks like a valid WAL file,
-        # including partial ones.
+        # including partial ones and history files.
         # Anything else is treated like an error/anomaly
         files = []
         skip = []
         errors = []
         for file_name in file_names:
-            if xlog.is_wal_file(file_name) and os.path.isfile(file_name):
-                files.append(file_name)
-            elif xlog.is_partial_file(file_name) and os.path.isfile(file_name):
+            if not os.path.isfile(file_name):
+                errors.append(file_name)
+            elif xlog.is_partial_file(file_name):
                 skip.append(file_name)
+            elif xlog.is_any_xlog_file(file_name):
+                files.append(file_name)
             else:
                 errors.append(file_name)
         # In case of more than a partial file, keep the last
         # and treat the rest as errors
         if len(skip) > 1:
             errors.extend(skip[:-1])
+            _logger.warning('Multiple partial files found for server %s: %s' %
+                            (self.config.name,
+                             ", ".join([os.path.basename(f) for f in errors])))
             skip = skip[-1:]
 
         # Keep the last full WAL file in case no partial file is present
@@ -658,7 +816,9 @@ class StreamingWalArchiver(WalArchiver):
 
         # Build the list of WalFileInfo
         wal_files = [WalFileInfo.from_file(f, compression=None) for f in files]
-        return WalArchiverBatch(wal_files, errors=errors, skip=skip)
+        return WalArchiverQueue(wal_files,
+                                batch_size=batch_size,
+                                errors=errors, skip=skip)
 
     def check(self, check_strategy):
         """
@@ -676,8 +836,11 @@ class StreamingWalArchiver(WalArchiver):
             remote_status['pg_receivexlog_installed'])
         hint = None
         if not remote_status['pg_receivexlog_compatible']:
+            pg_version = 'Unknown'
+            if self.server.streaming is not None:
+                pg_version = self.server.streaming.server_txt_version
             hint = "PostgreSQL version: %s, pg_receivexlog version: %s" % (
-                self.server.streaming.server_txt_version,
+                pg_version,
                 remote_status['pg_receivexlog_version']
             )
         check_strategy.result(
@@ -697,3 +860,41 @@ class StreamingWalArchiver(WalArchiver):
             check_strategy.result(
                 self.config.name, 'receive-wal running', False,
                 'See the Barman log file for more details')
+
+    def _is_synchronous(self):
+        """
+        Check if receive-wal process is eligible for synchronous replication
+
+        The receive-wal process is eligible for synchronous replication
+        if `synchronous_standby_names` is configured and contains
+        the value of `streaming_archiver_name`
+
+        :rtype: bool
+        """
+        # Nothing to do if postgres connection is not working
+        postgres = self.server.postgres
+        if postgres is None or postgres.server_txt_version is None:
+            return None
+
+        # Check if synchronous WAL streaming can be enabled
+        # by peeking 'synchronous_standby_names'
+        postgres_status = postgres.get_remote_status()
+        syncnames = postgres_status['synchronous_standby_names']
+        _logger.debug("Look for '%s' in "
+                      "'synchronous_standby_names': %s",
+                      self.config.streaming_archiver_name, syncnames)
+        # The receive-wal process is eligible for synchronous replication
+        # if `synchronous_standby_names` is configured and contains
+        # the value of `streaming_archiver_name`
+        synchronous = (syncnames and
+                       self.config.streaming_archiver_name in syncnames)
+        _logger.info('Synchronous WAL streaming for %s: %s',
+                     self.config.streaming_archiver_name,
+                     synchronous)
+        return synchronous
+
+    def status(self):
+        """
+        Set additional status info - invoked by Server.status()
+        """
+        # TODO: Add status information for WAL streaming
